package com.lacunasoftware.flutter.crosspki;

import java.util.ArrayList;
import java.util.Arrays;

public class DigestAlgorithm {

    public static final DigestAlgorithm MD5 = new DigestAlgorithm("MD5", "1.2.840.113549.2.5");
    public static final DigestAlgorithm SHA1 = new DigestAlgorithm("SHA-1", "1.3.14.3.2.26");
    public static final DigestAlgorithm SHA224 = new DigestAlgorithm("SHA-224", "2.16.840.1.101.3.4.2.4");
    public static final DigestAlgorithm SHA256 = new DigestAlgorithm("SHA-256", "2.16.840.1.101.3.4.2.1");
    public static final DigestAlgorithm SHA384 = new DigestAlgorithm("SHA-384", "2.16.840.1.101.3.4.2.2");
    public static final DigestAlgorithm SHA512 = new DigestAlgorithm("SHA-512", "2.16.840.1.101.3.4.2.3");

    private static final ArrayList<DigestAlgorithm> allAlgorithms = new ArrayList<>(Arrays.asList(MD5, SHA1, SHA224, SHA256, SHA384, SHA512));

    private String name;
    private String oid;

    private DigestAlgorithm(String name, String oid) {
        this.name = name;
        this.oid = oid;
    }

    public String getName() {
        return name;
    }

    public String getOid() {
        return oid;
    }

    public String getWithSignatureAlgorithm(String signatureAlgorithm) {
        return getName().replace("-", "") + "with" + signatureAlgorithm;
    }

    @Override
    public String toString() {
        return String.format("%s (%s)", getName(), getOid());
    }

    public static DigestAlgorithm parse(String digestAlgorithm) throws Exception {
        for (DigestAlgorithm alg : allAlgorithms) {
            if (alg.getOid().equals(digestAlgorithm)) {
                return alg;
            }

            if (getCanonName(alg.getName()).equals(getCanonName(digestAlgorithm))) {
                return alg;
            }
        }

        throw new CrossPkiException(ErrorCodes.ALGORITHM_NOT_SUPPORTED, "Unknown digest algorithm: " + digestAlgorithm);
    }

    private static String getCanonName(String str) {
        return str.toLowerCase().trim().replace("-", "").replace(" ", "");
    }
}
