package com.lacunasoftware.flutter.crosspki;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.HashMap;

public class DigestInfo {

    private static final HashMap<String, byte[]> digestIds = new HashMap<String, byte[]>() {{
		put(DigestAlgorithm.MD5.getOid(), new byte[] { 0x30, 0x20, 0x30, 0x0c, 0x06, 0x08, 0x2a, (byte)0x86, 0x48, (byte)0x86, (byte)0xf7, 0x0d, 0x02, 0x05, 0x05, 0x00, 0x04, 0x10 });
        put(DigestAlgorithm.SHA1.getOid(), new byte[] { 0x30, 0x21, 0x30, 0x09, 0x06, 0x05, 0x2b, 0x0e, 0x03, 0x02, 0x1a, 0x05, 0x00, 0x04, 0x14 });
        put(DigestAlgorithm.SHA256.getOid(), new byte[] { 0x30, 0x31, 0x30, 0x0d, 0x06, 0x09, 0x60, (byte)0x86, 0x48, 0x01, 0x65, 0x03, 0x04, 0x02, 0x01, 0x05, 0x00, 0x04, 0x20 });
        put(DigestAlgorithm.SHA384.getOid(), new byte[] { 0x30, 0x41, 0x30, 0x0d, 0x06, 0x09, 0x60, (byte)0x86, 0x48, 0x01, 0x65, 0x03, 0x04, 0x02, 0x02, 0x05, 0x00, 0x04, 0x30 });
        put(DigestAlgorithm.SHA512.getOid(), new byte[] { 0x30, 0x51, 0x30, 0x0d, 0x06, 0x09, 0x60, (byte)0x86, 0x48, 0x01, 0x65, 0x03, 0x04, 0x02, 0x03, 0x05, 0x00, 0x04, 0x40 });
    }};

    public static byte[] encode(DigestAlgorithm digestAlgorithm, byte[] digestValue) throws IOException, CrossPkiException {
        try (ByteArrayOutputStream ms = new ByteArrayOutputStream()) {
            if (!digestIds.containsKey(digestAlgorithm.getOid())) {
                throw new CrossPkiException(ErrorCodes.ALGORITHM_NOT_SUPPORTED, "Digest algorithm not supported: " + digestAlgorithm.toString());
            }
            byte[] id = digestIds.get(digestAlgorithm.getOid());
            ms.write(id, 0, id.length);
            ms.write(digestValue, 0, digestValue.length);
            return ms.toByteArray();
        }
    }
}
