package com.lacunasoftware.flutter.crosspki;

import android.os.Build;
import android.util.Base64;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

import javax.security.auth.x500.X500Principal;

public class PkiUtil {

	public static byte[] computeDigest(DigestAlgorithm digestAlgorithm, byte[] data) throws NoSuchAlgorithmException {
		MessageDigest md = MessageDigest.getInstance(digestAlgorithm.getName());
		return md.digest(data);
	}

	public static String getCommonName(X500Principal principal) {
		String dn = principal.getName();
		if (!dn.contains("CN=")) {
			return null;
		}
		String cn = dn.substring(dn.indexOf("CN="));
		return cn.contains(",")
			? cn.substring(3, cn.indexOf(","))
			: cn.substring(3);
	}

	public static X509Certificate decodeCertificate(byte[] content) throws IOException, CertificateException {
		try (ByteArrayInputStream inputStream = new ByteArrayInputStream(content)) {
			CertificateFactory certFactory = CertificateFactory.getInstance("X.509");
			return (X509Certificate) certFactory.generateCertificate(inputStream);
		}
	}

    public static X509Certificate decodeCertificate(String base64) throws IOException, CertificateException {
        if (android.os.Build.VERSION.SDK_INT < android.os.Build.VERSION_CODES.FROYO) {
            return null;
        }
        try (ByteArrayInputStream inputStream = new ByteArrayInputStream(Base64.decode(base64, Base64.DEFAULT))) {
            CertificateFactory certFactory = CertificateFactory.getInstance("X.509");
            return (X509Certificate) certFactory.generateCertificate(inputStream);
        }
    }
}