package com.lacunasoftware.flutter.crosspki;

import android.annotation.SuppressLint;
import android.os.Build;
import android.util.Base64;

import java.security.cert.X509Certificate;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.UUID;

public class Util {

	public static Map<String, Object> toMap(X509Certificate certificate) throws Exception {
		HashMap<String, Object> map = new HashMap<>();
		map.put("subjectDisplayName", PkiUtil.getCommonName(certificate.getSubjectX500Principal()));
		map.put("issuerDisplayName", PkiUtil.getCommonName(certificate.getIssuerX500Principal()));
		map.put("subjectName", certificate.getSubjectX500Principal().getName());
		map.put("issuerName", certificate.getIssuerX500Principal().getName());
		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
			map.put("validityStart", certificate.getNotBefore().toInstant().toString());
			map.put("validityEnd", certificate.getNotAfter().toInstant().toString());

		} else {
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'", Locale.UK);
			map.put("validityStart", sdf.format(certificate.getNotBefore()));
			map.put("validityEnd", sdf.format(certificate.getNotAfter()));
		}

		String thumbprint = base64Encode(PkiUtil.computeDigest(DigestAlgorithm.SHA256, certificate.getEncoded()));
		map.put("thumbprint", thumbprint);
		map.put("encoded", certificate.getEncoded());

		return map;
	}

	public static ArrayList<Map<String, Object>> toMap(ArrayList<X509Certificate> certificates) throws Exception {
		ArrayList<Map<String, Object>> map = new ArrayList<>();
		for (X509Certificate cert : certificates) {
			map.add(toMap(cert));
		}
		return map;
	}

    public static HashMap<String, Object> toMap(MobileKeyPair keyPair) {
	    HashMap<String, Object> map = new HashMap<>();
	    map.put("keyId", keyPair.getKeyId());
	    map.put("publicKeyEncoded", keyPair.getPublicKey().getEncoded());
	    return map;
    }

    @SuppressLint("NewApi")
	public static String base64Encode(byte[] content) {
		return Base64.encodeToString(content, Base64.DEFAULT);
	}

	@SuppressLint("NewApi")
	public static byte[] base64Decode(String base64) {
		return Base64.decode(base64, Base64.DEFAULT);
	}

	public static String createUniqueId() {
		return UUID.randomUUID().toString().replace("-", "");
	}
}